define(['./make_', '../array/find', '../object/find'], function(make, arrFind, objFind) {

    /**
     * Find value that returns true on iterator check.
     */
    return make(arrFind, objFind);

});
