module.exports = {
    GitFs: require('./GitFsResolver'),
    GitRemote: require('./GitRemoteResolver'),
    GitHub: require('./GitHubResolver'),
    Svn: require('./SvnResolver'),
    Fs: require('./FsResolver'),
    Url: require('./UrlResolver')
};
