exports.single = {
	topLeft: '┌',
	topRight: '┐',
	bottomRight: '┘',
	bottomLeft: '└',
	vertical: '│',
	horizontal: '─'
};

exports.double = {
	topLeft: '╔',
	topRight: '╗',
	bottomRight: '╝',
	bottomLeft: '╚',
	vertical: '║',
	horizontal: '═'
};

exports.round = {
	topLeft: '╭',
	topRight: '╮',
	bottomRight: '╯',
	bottomLeft: '╰',
	vertical: '│',
	horizontal: '─'
};

// 1st: top and bottom, 2nd: left and right (as in CSS shorthands)
exports['single-double'] = {
	topLeft: '╓',
	topRight: '╖',
	bottomRight: '╜',
	bottomLeft: '╙',
	vertical: '║',
	horizontal: '─'
};

exports['double-single'] = {
	topLeft: '╒',
	topRight: '╕',
	bottomRight: '╛',
	bottomLeft: '╘',
	vertical: '│',
	horizontal: '═'
};
