/**@license
 * mout v1.0.0 | http://moutjs.com | MIT license
 */
define(function(require){

//automatically generated, do not edit!
//run `node build` instead
return {
    'VERSION' : '1.0.0',
    'array' : require('./array'),
    'collection' : require('./collection'),
    'date' : require('./date'),
    'function' : require('./function'),
    'lang' : require('./lang'),
    'math' : require('./math'),
    'number' : require('./number'),
    'object' : require('./object'),
    'queryString' : require('./queryString'),
    'random' : require('./random'),
    'string' : require('./string'),
    'time' : require('./time'),
    'fn' : require('./function')
};

});
