
    // pt-BR (Brazillian Portuguese)
    module.exports = {
        "am" : "",
        "pm" : "",

        "x": "%d/%m/%y",
        "X": "%H:%M:%S",
        "c": "%a %d %b %Y %H:%M:%S %Z",

        "months" : [
            "Janeiro",
            "Fevereiro",
            "Março",
            "Abril",
            "Maio",
            "Junho",
            "Julho",
            "Agosto",
            "Setembro",
            "Outubro",
            "Novembro",
            "Dezembro"
        ],

        "months_abbr" : [
            "Jan",
            "Fev",
            "Mar",
            "Abr",
            "Mai",
            "Jun",
            "Jul",
            "Ago",
            "Set",
            "Out",
            "Nov",
            "Dez"
        ],

        "days" : [
            "Domingo",
            "Segunda",
            "Terça",
            "Quarta",
            "Quinta",
            "Sexta",
            "Sábado"
        ],

        "days_abbr" : [
            "Dom",
            "Seg",
            "Ter",
            "Qua",
            "Qui",
            "Sex",
            "Sáb"
        ]
    };

